@@ -164,7 +164,7 @@ def test_tuning_to_A4(tuning, bins_per_octave, A4):
 )
 @pytest.mark.parametrize("accidental", ["", "#", "b", "!"])
 @pytest.mark.parametrize("round_midi", [False, True])
-def test_note_to_midi(tuning, accidental, octave, round_midi):
+def test_note_to_midi(tuning, accidental, octave, round_midi: bool):
 
     note = "C{:s}".format(accidental)
 
@@ -216,7 +216,7 @@ def test_note_to_midi_badnote():
 )
 @pytest.mark.parametrize("accidental", ["", "#", "b", "!"])
 @pytest.mark.parametrize("round_midi", [False, True])
-def test_note_to_hz(tuning, octave, accidental, round_midi):
+def test_note_to_hz(tuning, octave, accidental, round_midi: bool):
 
     note = "A{:s}".format(accidental)
 
@@ -293,7 +293,10 @@ def test_hz_to_midi():
 )
 def test_hz_to_note(hz, note, octave, cents):
     note_out = librosa.hz_to_note(hz, octave=octave, cents=cents)
-    assert note_out == note
+    if np.isscalar(hz):
+        assert note_out == note
+    else:
+        assert np.all(note_out == note)
 
 
 @pytest.mark.xfail(raises=librosa.ParameterError)
@@ -531,14 +534,16 @@ def test_blocks_to_frames(blocks, block_length):
     assert np.allclose(frames, block_length * np.asanyarray(blocks))
 
     # Check dtype
-    assert np.issubdtype(frames.dtype, np.int)
+    assert np.issubdtype(frames.dtype, int)
 
 
 @pytest.mark.parametrize("blocks", [0, 1, [10, 20]])
 @pytest.mark.parametrize("block_length", [1, 4, 8])
 @pytest.mark.parametrize("hop_length", [1, 512])
 def test_blocks_to_samples(blocks, block_length, hop_length):
-    samples = librosa.blocks_to_samples(blocks, block_length=block_length, hop_length=hop_length)
+    samples = librosa.blocks_to_samples(
+        blocks, block_length=block_length, hop_length=hop_length
+    )
 
     # Check shape
     assert samples.ndim == np.asarray(blocks).ndim
@@ -548,7 +553,7 @@ def test_blocks_to_samples(blocks, block_length, hop_length):
     assert np.allclose(samples, np.asanyarray(blocks) * hop_length * block_length)
 
     # Check dtype
-    assert np.issubdtype(samples.dtype, np.int)
+    assert np.issubdtype(samples.dtype, int)
 
 
 @pytest.mark.parametrize("blocks", [0, 1, [10, 20]])
@@ -556,7 +561,9 @@ def test_blocks_to_samples(blocks, block_length, hop_length):
 @pytest.mark.parametrize("hop_length", [1, 512])
 @pytest.mark.parametrize("sr", [22050, 44100])
 def test_blocks_to_time(blocks, block_length, hop_length, sr):
-    times = librosa.blocks_to_time(blocks, block_length=block_length, hop_length=hop_length, sr=sr)
+    times = librosa.blocks_to_time(
+        blocks, block_length=block_length, hop_length=hop_length, sr=sr
+    )
 
     # Check shape
     assert times.ndim == np.asarray(blocks).ndim
@@ -568,7 +575,7 @@ def test_blocks_to_time(blocks, block_length, hop_length, sr):
     )
 
     # Check dtype
-    assert np.issubdtype(times.dtype, np.float)
+    assert np.issubdtype(times.dtype, float)
 
 
 @pytest.mark.parametrize("abbr", [False, True])
@@ -578,7 +585,9 @@ def test_blocks_to_time(blocks, block_length, hop_length, sr):
 @pytest.mark.parametrize("Sa", [12])
 def test_midi_to_svara_h(midi, Sa, abbr, octave, unicode):
 
-    svara = librosa.midi_to_svara_h(midi, Sa=Sa, abbr=abbr, octave=octave, unicode=unicode)
+    svara = librosa.midi_to_svara_h(
+        midi, Sa=Sa, abbr=abbr, octave=octave, unicode=unicode
+    )
 
     svara = np.asarray(svara)
     assert len(svara) == len(midi)
@@ -704,7 +713,9 @@ def test_midi_to_svara_c(midi, Sa, mela, abbr, octave, unicode):
     ],
 )
 def test_hz_to_svara_c(freq, Sa, mela, abbr, octave, unicode, result):
-    s = librosa.hz_to_svara_c(freq, Sa=Sa, mela=mela, abbr=abbr, octave=octave, unicode=unicode)
+    s = librosa.hz_to_svara_c(
+        freq, Sa=Sa, mela=mela, abbr=abbr, octave=octave, unicode=unicode
+    )
     assert s == result
 
 
