@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 # -*- encoding: utf-8 -*-
 
-from __future__ import print_function
 import warnings
 import numpy as np
 
@@ -426,7 +425,7 @@ def poly_freq(request):
     freq = librosa.fft_frequencies()
 
     if request.param in (1, 2):
-        return freq ** request.param
+        return freq**request.param
 
     elif request.param == -1:
         return np.cumsum(np.abs(np.random.randn(1 + 2048 // 2)), axis=0)
@@ -443,7 +442,7 @@ def poly_S(poly_coeffs, poly_freq):
 
     S = np.zeros_like(poly_freq)
     for i, c in enumerate(poly_coeffs):
-        S += c * poly_freq ** i
+        S += c * poly_freq**i
 
     return S.reshape((poly_freq.shape[0], -1))
 
@@ -663,6 +662,9 @@ def test_fourier_tempogram_fail_noinput():
 
 
 @pytest.mark.parametrize("hop_length", [512, 1024])
+@pytest.mark.filterwarnings(
+    "ignore:n_fft=.*is too large"
+)  # our test signal is short, but this is fine here
 def test_fourier_tempogram_audio(y_ex, hop_length):
     y, sr = y_ex
     oenv = librosa.onset.onset_strength(y=y, sr=sr, hop_length=hop_length)
@@ -832,7 +834,7 @@ def test_mel_to_stft(power, dtype, n_fft):
     assert stft.shape[0] == 1 + n_fft // 2
 
     # Check that the approximation is good in RMSE terms
-    assert np.sqrt(np.mean((mel_basis.dot(stft ** power) - mels) ** 2)) <= 5e-2
+    assert np.sqrt(np.mean((mel_basis.dot(stft**power) - mels) ** 2)) <= 5e-2
 
 
 def test_mel_to_audio():
@@ -863,7 +865,7 @@ def test_mfcc_to_mel(y, n_mfcc, n_mels, dct_type, lifter):
     if lifter < 0:
         with pytest.raises(librosa.ParameterError):
             librosa.feature.inverse.mfcc_to_mel(
-                mfcc * 10 ** 3, n_mels=n_mels, dct_type=dct_type, lifter=lifter
+                mfcc * 10**3, n_mels=n_mels, dct_type=dct_type, lifter=lifter
             )
 
     # check no lifter computations
@@ -881,9 +883,9 @@ def test_mfcc_to_mel(y, n_mfcc, n_mels, dct_type, lifter):
 
     # check that runtime warnings are triggered when appropriate
     elif lifter == 2:
-        with pytest.warns(UserWarning):
+        with pytest.warns((UserWarning, RuntimeWarning)):
             librosa.feature.inverse.mfcc_to_mel(
-                mfcc * 10 ** 3, n_mels=n_mels, dct_type=dct_type, lifter=lifter
+                mfcc * 10**3, n_mels=n_mels, dct_type=dct_type, lifter=lifter
             )
 
     # check if mfcc_to_mel works correctly with lifter
@@ -927,3 +929,97 @@ def test_mfcc_to_audio(y, n_mfcc, n_mels, dct_type, lifter):
 
     # And that it's valid audio
     assert librosa.util.valid_audio(y_inv)
+
+
+def test_chroma_vqt_bpo(y_ex):
+    # Test that bins per octave is properly overridden in chroma
+    y, sr = y_ex
+    chroma = librosa.feature.chroma_vqt(
+        y=y, sr=sr, intervals=[1, 1.25, 1.5], bins_per_octave=12
+    )
+
+    assert chroma.shape[0] == 3
+
+    chroma2 = librosa.feature.chroma_vqt(
+        y=y, sr=sr, intervals="equal", bins_per_octave=12
+    )
+
+    assert chroma2.shape[0] == 12
+
+
+def test_chroma_vqt_threshold(y_ex):
+
+    y, sr = y_ex
+
+    c1 = librosa.feature.chroma_vqt(y=y, sr=sr, intervals="pythagorean")
+    c2 = librosa.feature.chroma_vqt(y=y, sr=sr, intervals="pythagorean", threshold=1)
+
+    # Check that all thresholded points are zero
+    assert np.allclose(c2[c2 < c1], 0)
+    # Check that all non-thresholded points match
+    assert np.all(c2 <= c1)
+
+
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_chroma_vqt_noinput():
+    librosa.feature.chroma_vqt(y=None, V=None, intervals="ji3")
+
+
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_chroma_cqt_noinput():
+    librosa.feature.chroma_cqt(y=None, C=None)
+
+
+def test_tempogram_ratio_factors():
+    # Testing with synthetic data and specific factors
+
+    # tg is [0, 1, 2, 3, 4]  for each frame
+    tg = np.multiply.outer(np.arange(5), np.ones(4))
+    # frequencies are [1, 2, 4, 8, 16]
+    freqs = 2 ** np.arange(5)
+    factors = np.array([1, 2, 4])
+    bpm = np.array([4, 2, 1, 1.5])
+
+    tgr = librosa.feature.tempogram_ratio(tg=tg, freqs=freqs, factors=factors, bpm=bpm)
+
+    # frame 0: bpm = 4, factors are [1, 2, 4] => [4, 8, 16] => values 2 3 4
+    assert np.allclose(tgr[:, 0], [2, 3, 4])
+    # frame 1: bpm = 2, factors are [1, 2, 4] => [2, 4, 8] => values [0, 2, 3]
+    assert np.allclose(tgr[:, 1], [1, 2, 3])
+    # frame 2: bpm = 1, factors are [1, 2, 4] => [1, 2, 4] => values [0, 1, 2]
+    assert np.allclose(tgr[:, 2], [0, 1, 2])
+    # frame 3: bpm = 1.5, factors are [1, 2, 4] => [1.5, 3, 6] => values
+    # [0.5, 1.5, 2.5]
+    assert np.allclose(tgr[:, 3], [0.5, 1.5, 2.5])
+
+
+@pytest.fixture(scope="module")
+def tg_ex(y_ex):
+    y, sr = y_ex
+    return librosa.feature.tempogram(y=y, sr=sr)
+
+
+def test_tempogram_ratio_aggregate(y_ex, tg_ex):
+    # Verify that aggregation does its job
+    _, sr = y_ex
+    tgr1 = librosa.feature.tempogram_ratio(sr=sr, tg=tg_ex, aggregate=None)
+    tgr2 = librosa.feature.tempogram_ratio(sr=sr, tg=tg_ex, aggregate=np.median)
+    assert np.allclose(np.median(tgr1, axis=-1), tgr2)
+
+
+def test_tempogram_ratio_with_tg(y_ex, tg_ex):
+    # Verify equivalent behavior with/without pre-computed tempogram
+    y, sr = y_ex
+
+    tgr1 = librosa.feature.tempogram_ratio(y=y, sr=sr)
+    tgr2 = librosa.feature.tempogram_ratio(tg=tg_ex, sr=sr)
+
+    assert np.allclose(tgr1, tgr2)
+
+
+def test_tempogram_ratio_with_bpm(y_ex, tg_ex):
+    y, sr = y_ex
+    tempo = librosa.feature.tempo(tg=tg_ex, sr=sr, aggregate=None)
+    tgr1 = librosa.feature.tempogram_ratio(tg=tg_ex, sr=sr, bpm=None)
+    tgr2 = librosa.feature.tempogram_ratio(tg=tg_ex, sr=sr, bpm=tempo)
+    assert np.allclose(tgr1, tgr2)
