@@ -3,25 +3,25 @@
 #  unit tests for multi-channel functionality
 #
 
-from __future__ import print_function
-
 # Disable cache
 import os
 
 try:
     os.environ.pop("LIBROSA_CACHE_DIR")
-except:
+except KeyError:
     pass
 
 import librosa
 import glob
 import numpy as np
 import scipy.io
+import scipy.stats
 import pytest
 import warnings
 from unittest import mock
+from typing import List, Union
 
-from contextlib2 import nullcontext as dnr
+from contextlib import nullcontext as dnr
 from test_core import srand
 
 
@@ -45,10 +45,23 @@ def tfr_multi(y_multi):
 
 @pytest.mark.parametrize("aggregate", [None, np.mean, np.sum])
 @pytest.mark.parametrize(
-    "ndim,axis", [(1, 0), (1, -1), (2, 0), (2, 1), (2, -1), (3, 0), (3, 2), (3, -1), (4, 0), (4, 3), (4, -1)]
+    "ndim,axis",
+    [
+        (1, 0),
+        (1, -1),
+        (2, 0),
+        (2, 1),
+        (2, -1),
+        (3, 0),
+        (3, 2),
+        (3, -1),
+        (4, 0),
+        (4, 3),
+        (4, -1),
+    ],
 )
-def test_sync_multi(aggregate, ndim, axis):
-    data = np.ones([6] * ndim, dtype=np.float)
+def test_sync_multi(aggregate, ndim: int, axis: int):
+    data = np.ones([6] * ndim, dtype=float)
 
     # Make some slices that don't fill the entire dimension
     slices = [slice(1, 3), slice(3, 4)]
@@ -64,16 +77,16 @@ def test_sync_multi(aggregate, ndim, axis):
     assert s_test == s_orig
 
     # The first slice will sum to 2 and have mean 1
-    idx = [slice(None)] * ndim
+    idx: List[Union[slice, int]] = [slice(None)] * ndim
     idx[axis] = 0
     if aggregate is np.sum:
-        assert np.allclose(dsync[idx], 2)
+        assert np.allclose(dsync[tuple(idx)], 2)
     else:
-        assert np.allclose(dsync[idx], 1)
+        assert np.allclose(dsync[tuple(idx)], 1)
 
     # The second slice will sum to 1 and have mean 1
     idx[axis] = 1
-    assert np.allclose(dsync[idx], 1)
+    assert np.allclose(dsync[tuple(idx)], 1)
 
 
 def test_stft_multi(y_multi):
@@ -165,37 +178,22 @@ def test_tempo_multi(y_multi):
     sr = 22050
     tempi = [78, 128]
 
-    y = np.zeros((2, 20*sr))
+    y = np.zeros((2, 20 * sr))
 
     delay = [librosa.time_to_samples(60 / tempo, sr=sr).item() for tempo in tempi]
-    y[0,::delay[0]] = 1
-    y[1,::delay[1]] = 1
+    y[0, :: delay[0]] = 1
+    y[1, :: delay[1]] = 1
 
-    t = librosa.beat.tempo(
-        y=y,
-        sr=sr,
-        hop_length=512,
-        ac_size=4,
-        aggregate=np.mean,
-        prior=None
+    t = librosa.feature.tempo(
+        y=y, sr=sr, hop_length=512, ac_size=4, aggregate=np.mean, prior=None
     )
 
-    t0 = librosa.beat.tempo(
-        y=y[0],
-        sr=sr,
-        hop_length=512,
-        ac_size=4,
-        aggregate=np.mean,
-        prior=None
+    t0 = librosa.feature.tempo(
+        y=y[0], sr=sr, hop_length=512, ac_size=4, aggregate=np.mean, prior=None
     )
 
-    t1 = librosa.beat.tempo(
-        y=y[1],
-        sr=sr,
-        hop_length=512,
-        ac_size=4,
-        aggregate=np.mean,
-        prior=None
+    t1 = librosa.feature.tempo(
+        y=y[1], sr=sr, hop_length=512, ac_size=4, aggregate=np.mean, prior=None
     )
 
     # Check each channel
@@ -299,6 +297,8 @@ def test_griffinlim_multi(y_multi):
 
 @pytest.mark.parametrize("scale", [False, True])
 @pytest.mark.parametrize("res_type", [None, "polyphase"])
+# The following warning is fine in context here
+@pytest.mark.filterwarnings("ignore:Support for VQT with res_type=None")
 def test_cqt_multi(y_multi, scale, res_type):
 
     y, sr = y_multi
@@ -318,6 +318,7 @@ def test_cqt_multi(y_multi, scale, res_type):
 
 @pytest.mark.parametrize("scale", [False, True])
 @pytest.mark.parametrize("res_type", [None, "polyphase"])
+@pytest.mark.filterwarnings("ignore:Support for VQT with res_type=None")
 def test_hybrid_cqt_multi(y_multi, scale, res_type):
 
     y, sr = y_multi
@@ -351,9 +352,9 @@ def test_icqt_multi(y_multi, scale, length):
     if length is not None:
         assert yboth.shape[-1] == length
 
-    # Check each channel
-    assert np.allclose(yboth[0], y0)
-    assert np.allclose(yboth[1], y1)
+    # Check each channel - slightly relaxed tolerance here
+    assert np.allclose(yboth[0], y0, atol=1e-6), np.max(np.abs(yboth[0] - y0))
+    assert np.allclose(yboth[1], y1, atol=1e-6), np.max(np.abs(yboth[1] - y1))
 
     # Check that they're not the same
     assert not np.allclose(yboth[0], yboth[1])
@@ -395,7 +396,7 @@ def test_spectral_centroid_multi_variable(s_multi):
 
     S, sr = s_multi
 
-    freq = np.random.randn(*S.shape)
+    freq = np.asarray(np.random.randn(*S.shape))
 
     # compare each channel
     C0 = librosa.feature.spectral_centroid(sr=sr, freq=freq[0], S=S[0])
@@ -431,7 +432,7 @@ def test_spectral_bandwidth_multi(s_multi):
 def test_spectral_bandwidth_multi_variable(s_multi):
     S, sr = s_multi
 
-    freq = np.random.randn(*S.shape)
+    freq = np.asarray(np.random.randn(*S.shape))
 
     # compare each channel
     C0 = librosa.feature.spectral_bandwidth(sr=sr, freq=freq[0], S=S[0])
@@ -485,7 +486,7 @@ def test_spectral_rolloff_multi(s_multi):
 def test_spectral_rolloff_multi_variable(s_multi):
     S, sr = s_multi
 
-    freq = np.random.randn(*S.shape)
+    freq = np.asarray(np.random.randn(*S.shape))
 
     # compare each channel
     C0 = librosa.feature.spectral_rolloff(sr=sr, freq=freq[0], S=S[0])
@@ -531,6 +532,8 @@ def test_poly_multi_static(s_multi):
     assert not np.allclose(P0, P1)
 
 
+# Not worried about polyfit conditioning for this test
+@pytest.mark.filterwarnings("ignore:Polyfit may be poorly conditioned")
 def test_poly_multi_varying(tfr_multi):
 
     # Get some time-varying frequencies
@@ -755,13 +758,20 @@ def test_interp_harmonics_multi_static(s_multi):
     assert not np.allclose(H0, H1)
 
 
+# Not worried about this warning here
+@pytest.mark.filterwarnings("ignore:Frequencies are not unique")
 def test_interp_harmonics_multi_vary(tfr_multi):
     times, freqs, mags = tfr_multi
 
-    # Force slinear mode here to deal with non-unique frequencies
-    Hall = librosa.interp_harmonics(mags, freqs=freqs, harmonics=[0.5, 1, 2], kind="slinear")
-    H0 = librosa.interp_harmonics(mags[0], freqs=freqs[0], harmonics=[0.5, 1, 2], kind="slinear")
-    H1 = librosa.interp_harmonics(mags[1], freqs=freqs[1], harmonics=[0.5, 1, 2], kind="slinear")
+    Hall = librosa.interp_harmonics(
+        mags, freqs=freqs, harmonics=[0.5, 1, 2], kind="nearest"
+    )
+    H0 = librosa.interp_harmonics(
+        mags[0], freqs=freqs[0], harmonics=[0.5, 1, 2], kind="nearest"
+    )
+    H1 = librosa.interp_harmonics(
+        mags[1], freqs=freqs[1], harmonics=[0.5, 1, 2], kind="nearest"
+    )
 
     assert np.allclose(Hall[0], H0)
     assert np.allclose(Hall[1], H1)
@@ -806,6 +816,8 @@ def test_salience_multi_static(s_multi, filter_peaks):
 
 
 @pytest.mark.parametrize("filter_peaks", [False, True])
+# Not worried about this warning here
+@pytest.mark.filterwarnings("ignore:Frequencies are not unique")
 def test_salience_multi_dynamic(tfr_multi, filter_peaks):
     times, freqs, S = tfr_multi
 
@@ -813,7 +825,7 @@ def test_salience_multi_dynamic(tfr_multi, filter_peaks):
         S,
         freqs=freqs,
         harmonics=[0.5, 1, 2, 3],
-        kind="slinear",
+        kind="nearest",
         filter_peaks=filter_peaks,
         fill_value=0,
     )
@@ -821,7 +833,7 @@ def test_salience_multi_dynamic(tfr_multi, filter_peaks):
         S[0],
         freqs=freqs[0],
         harmonics=[0.5, 1, 2, 3],
-        kind="slinear",
+        kind="nearest",
         filter_peaks=filter_peaks,
         fill_value=0,
     )
@@ -829,7 +841,7 @@ def test_salience_multi_dynamic(tfr_multi, filter_peaks):
         S[1],
         freqs=freqs[1],
         harmonics=[0.5, 1, 2, 3],
-        kind="slinear",
+        kind="nearest",
         filter_peaks=filter_peaks,
         fill_value=0,
     )
@@ -877,7 +889,7 @@ def test_yin_multi(y_multi):
     assert not np.allclose(P0, P1)
 
 
-@pytest.mark.parametrize('ref', [None, 1.0])
+@pytest.mark.parametrize("ref", [None, 1.0])
 def test_piptrack_multi(s_multi, ref):
     S, sr = s_multi
 
@@ -899,7 +911,6 @@ def test_click_multi():
 
     yout = librosa.clicks(times=[0, 1, 2], sr=1000, click=click)
 
-    print(yout.shape)
     assert yout.shape[0] == click.shape[0]
 
     assert np.allclose(yout[..., :100], click)
@@ -912,25 +923,28 @@ def test_nnls_multi(s_multi):
     # Verify that a stereo melspectrogram can be reconstructed
     # for each channel individually
     S, sr = s_multi
-    S = S[...,:int(S.shape[-1]/2)]
+    S = S[..., : int(S.shape[-1] / 2)]
 
-    # multichannel  
-    mel_basis = librosa.filters.mel(sr=sr, n_fft=2*S.shape[-2]-1, n_mels=256)
-    M = np.einsum('...ft,mf->...mt', S, mel_basis)
-    print(M.shape, mel_basis.shape)
+    # multichannel
+    mel_basis = librosa.filters.mel(sr=sr, n_fft=2 * S.shape[-2] - 1, n_mels=256)
+    M = np.einsum("...ft,mf->...mt", S, mel_basis)
     S_recover = librosa.util.nnls(mel_basis, M)
 
     # channel 0
-    M0 = np.einsum('...ft,mf->...mt', S[0], mel_basis)
+    M0 = np.einsum("...ft,mf->...mt", S[0], mel_basis)
     S0_recover = librosa.util.nnls(mel_basis, M0)
 
     # channel 1
-    M1 = np.einsum('...ft,mf->...mt', S[1], mel_basis)
+    M1 = np.einsum("...ft,mf->...mt", S[1], mel_basis)
     S1_recover = librosa.util.nnls(mel_basis, M1)
 
     # Check each channel
-    assert np.allclose(S_recover[0], S0_recover, atol=1e-5, rtol=1e-5), np.max(np.abs(S_recover[0]-S0_recover))
-    assert np.allclose(S_recover[1], S1_recover, atol=1e-5, rtol=1e-5), np.max(np.abs(S_recover[1]-S1_recover))
+    assert np.allclose(S_recover[0], S0_recover, atol=1e-5, rtol=1e-5), np.max(
+        np.abs(S_recover[0] - S0_recover)
+    )
+    assert np.allclose(S_recover[1], S1_recover, atol=1e-5, rtol=1e-5), np.max(
+        np.abs(S_recover[1] - S1_recover)
+    )
 
     # Check that they're not both the same
     assert not np.allclose(S0_recover, S1_recover)
@@ -939,7 +953,7 @@ def test_nnls_multi(s_multi):
 # -- feature inversion tests
 @pytest.mark.parametrize("power", [1, 2])
 @pytest.mark.parametrize("n_fft", [1024, 2048])
-def test_mel_to_stft_multi(power,  n_fft):
+def test_mel_to_stft_multi(power, n_fft):
     srand()
 
     # Make a random mel spectrum, 4 frames
@@ -947,11 +961,11 @@ def test_mel_to_stft_multi(power,  n_fft):
 
     stft_orig = np.random.randn(2, n_fft // 2 + 1, 4) ** power
 
-    mels = np.einsum('...ft,mf->...mt', stft_orig, mel_basis)
+    mels = np.einsum("...ft,mf->...mt", stft_orig, mel_basis)
     stft = librosa.feature.inverse.mel_to_stft(mels, power=power, n_fft=n_fft)
-    mels0 = np.einsum('...ft,mf->...mt', stft_orig[0], mel_basis)
+    mels0 = np.einsum("...ft,mf->...mt", stft_orig[0], mel_basis)
     stft0 = librosa.feature.inverse.mel_to_stft(mels0, power=power, n_fft=n_fft)
-    mels1 = np.einsum('...ft,mf->...mt', stft_orig[1], mel_basis)
+    mels1 = np.einsum("...ft,mf->...mt", stft_orig[1], mel_basis)
     stft1 = librosa.feature.inverse.mel_to_stft(mels1, power=power, n_fft=n_fft)
 
     # Check each channel
@@ -992,7 +1006,6 @@ def test_mfcc_to_mel_multi(s_multi, n_mfcc, n_mels, dct_type):
     assert not np.allclose(mel_recover0, mel_recover1)
 
 
-
 def test_trim_multichannel(y_multi):
     y, sr = y_multi
 
@@ -1008,3 +1021,245 @@ def test_trim_multichannel(y_multi):
     # intersection of the individual intervals
     assert ival[0] == max(ival0[0], ival1[0])
     assert ival[1] == min(ival0[1], ival1[1])
+
+
+@pytest.mark.parametrize(
+    "res_type", ("scipy", "polyphase", "sinc_fastest", "kaiser_fast", "soxr_qq")
+)
+def test_resample_multichannel(y_multi, res_type):
+    # Test multi-channel resampling with all backends: scipy, samplerate, resampy, soxr
+    y, sr = y_multi
+
+    y_res = librosa.resample(y=y, orig_sr=sr, target_sr=sr // 2, res_type=res_type)
+    y0_res = librosa.resample(y=y[0], orig_sr=sr, target_sr=sr // 2, res_type=res_type)
+    y1_res = librosa.resample(y=y[1], orig_sr=sr, target_sr=sr // 2, res_type=res_type)
+
+    assert np.allclose(y_res[0], y0_res)
+    assert np.allclose(y_res[1], y1_res)
+    assert y_res[0].shape == y0_res.shape
+
+
+@pytest.mark.parametrize(
+    "res_type", ("scipy", "polyphase", "sinc_fastest", "kaiser_fast", "soxr_qq")
+)
+@pytest.mark.parametrize("x", [np.zeros((2, 2, 2, 22050))])
+def test_resample_highdim(x, res_type):
+    # Just run these to verify that it doesn't crash with ndim>2
+    y = librosa.resample(x, orig_sr=22050, target_sr=11025, res_type=res_type)
+
+
+@pytest.mark.parametrize(
+    "res_type", ("scipy", "polyphase", "sinc_fastest", "kaiser_fast", "soxr_qq")
+)
+@pytest.mark.parametrize(
+    "x, axis", [(np.zeros((2, 2, 2, 22050)), -1), (np.zeros((22050, 2, 3)), 0)]
+)
+def test_resample_highdim_axis(x, axis, res_type):
+    # Resample along the target axis
+    y = librosa.resample(
+        x, orig_sr=22050, target_sr=11025, axis=axis, res_type=res_type
+    )
+
+    # Verify that the target axis is the correct shape
+    assert y.shape[axis] == 11025
+    assert y.ndim == x.ndim
+
+
+@pytest.mark.parametrize("dynamic", [False, True])
+# Not worried about this warning here
+@pytest.mark.filterwarnings("ignore:Frequencies are not unique")
+def test_f0_harmonics(y_multi, dynamic):
+
+    y, sr = y_multi
+    Df, _, S = librosa.reassigned_spectrogram(y, sr=sr, fill_nan=True)
+    freqs = librosa.fft_frequencies(sr=sr)
+
+    harmonics = np.array([1, 2, 3])
+
+    f0 = 100 + 30 * np.random.random_sample(size=(S.shape[0], S.shape[-1]))
+
+    if dynamic:
+        out = librosa.f0_harmonics(S, freqs=Df, f0=f0, harmonics=harmonics)
+        out0 = librosa.f0_harmonics(S[0], freqs=Df[0], f0=f0[0], harmonics=harmonics)
+        out1 = librosa.f0_harmonics(S[1], freqs=Df[1], f0=f0[1], harmonics=harmonics)
+    else:
+        out = librosa.f0_harmonics(S, freqs=freqs, f0=f0, harmonics=harmonics)
+        out0 = librosa.f0_harmonics(S[0], freqs=freqs, f0=f0[0], harmonics=harmonics)
+        out1 = librosa.f0_harmonics(S[1], freqs=freqs, f0=f0[1], harmonics=harmonics)
+
+    assert np.allclose(out[0], out0)
+    assert np.allclose(out[1], out1)
+
+
+def test_peak_pick_multi():
+
+    x = np.random.randn(3, 1000) ** 2
+
+    pre_max = 5
+    post_max = 5
+    pre_avg = 10
+    post_avg = 10
+    wait = 10
+    delta = 0.5
+
+    pm = librosa.util.peak_pick(
+        x,
+        pre_max=pre_max,
+        post_max=post_max,
+        pre_avg=pre_avg,
+        post_avg=post_avg,
+        wait=wait,
+        delta=delta,
+        sparse=False,
+        axis=-1,
+    )
+
+    for i in range(x.shape[0]):
+        pmi = librosa.util.peak_pick(
+            x[i],
+            pre_max=pre_max,
+            post_max=post_max,
+            pre_avg=pre_avg,
+            post_avg=post_avg,
+            wait=wait,
+            delta=delta,
+            sparse=False,
+            axis=-1,
+        )
+        assert np.allclose(pmi, pm[i])
+
+
+def test_peak_pick_axis():
+
+    x = np.random.randn(100, 500) ** 2
+
+    pre_max = 5
+    post_max = 5
+    pre_avg = 10
+    post_avg = 10
+    wait = 10
+    delta = 0.5
+
+    peaks = librosa.util.peak_pick(
+        x,
+        pre_max=pre_max,
+        post_max=post_max,
+        pre_avg=pre_avg,
+        post_avg=post_avg,
+        wait=wait,
+        delta=delta,
+        sparse=False,
+        axis=-1,
+    )
+    peaks_t = librosa.util.peak_pick(
+        x.T,
+        pre_max=pre_max,
+        post_max=post_max,
+        pre_avg=pre_avg,
+        post_avg=post_avg,
+        wait=wait,
+        delta=delta,
+        sparse=False,
+        axis=0,
+    )
+
+    assert np.allclose(peaks_t.T, peaks)
+
+
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_peak_pick_multi_fail():
+
+    x = np.random.randn(3, 1000) ** 2
+
+    pre_max = 5
+    post_max = 5
+    pre_avg = 10
+    post_avg = 10
+    wait = 10
+    delta = 0.5
+
+    pm = librosa.util.peak_pick(
+        x,
+        pre_max=pre_max,
+        post_max=post_max,
+        pre_avg=pre_avg,
+        post_avg=post_avg,
+        wait=wait,
+        delta=delta,
+        sparse=True,
+        axis=-1,
+    )
+
+
+def test_onset_detect(y_multi):
+
+    # Verify that a stereo onset detection matches each channel individually
+    y, sr = y_multi
+
+    # Pre-compute the onset strength here using both channels to avoid any
+    # channel normalization issues if we were to process fully independently
+    oenv = librosa.onset.onset_strength(y=y, sr=sr)
+
+    D = librosa.onset.onset_detect(onset_envelope=oenv, sr=sr, sparse=False)
+    D0 = librosa.onset.onset_detect(onset_envelope=oenv[0], sr=sr, sparse=False)
+    D1 = librosa.onset.onset_detect(onset_envelope=oenv[1], sr=sr, sparse=False)
+
+    # Check each channel
+    assert np.allclose(D[0], D0)
+    assert np.allclose(D[1], D1)
+
+    # Check that they're not both the same
+    assert not np.allclose(D0, D1)
+
+
+@pytest.mark.parametrize("trim", [False, True])
+def test_beat_track_multi(y_multi, trim):
+    y, sr = y_multi
+
+    tempo, beats = librosa.beat.beat_track(y=y, sr=sr, trim=trim, sparse=False)
+    tempo0, beats0 = librosa.beat.beat_track(y=y[0], sr=sr, trim=trim, sparse=False)
+    tempo1, beats1 = librosa.beat.beat_track(y=y[1], sr=sr, trim=trim, sparse=False)
+
+    assert isinstance(tempo, np.ndarray)
+    assert np.allclose(tempo[0], tempo0)
+    assert np.allclose(tempo[1], tempo1)
+    assert np.allclose(beats[0], beats0)
+    assert np.allclose(beats[1], beats1)
+
+
+def test_beat_track_multi_bpm_scalar(y_multi):
+    y, sr = y_multi
+
+    tempo, beats = librosa.beat.beat_track(y=y, sr=sr, sparse=False, bpm=100)
+    tempo0, beats0 = librosa.beat.beat_track(y=y[0], sr=sr, sparse=False, bpm=100)
+    tempo1, beats1 = librosa.beat.beat_track(y=y[1], sr=sr, sparse=False, bpm=100)
+
+    assert np.isscalar(tempo)
+    assert np.allclose(tempo, tempo0)
+    assert np.allclose(tempo, tempo1)
+    assert np.allclose(beats[0], beats0)
+    assert np.allclose(beats[1], beats1)
+
+
+def test_beat_track_multi_bpm_vector(y_multi):
+    y, sr = y_multi
+
+    bpm = np.array([[150], [75]])
+    assert isinstance(bpm, np.ndarray)
+
+    tempo, beats = librosa.beat.beat_track(y=y, sr=sr, sparse=False, bpm=bpm)
+    tempo0, beats0 = librosa.beat.beat_track(y=y[0], sr=sr, sparse=False, bpm=bpm[0])
+    tempo1, beats1 = librosa.beat.beat_track(y=y[1], sr=sr, sparse=False, bpm=bpm[1])
+
+    assert isinstance(tempo, np.ndarray)
+    assert np.allclose(tempo, bpm)
+    assert np.allclose(tempo[0], tempo0)
+    assert np.allclose(tempo[1], tempo1)
+    assert np.allclose(beats[0], beats0)
+    assert np.allclose(beats[1], beats1)
+
+
+@pytest.mark.xfail(raises=librosa.ParameterError)
+def test_beat_track_multi_sparse(y_multi):
+    y, sr = y_multi
+    librosa.beat.beat_track(y=y, sr=sr, sparse=True)
